'''
Created on 18 Jan 2011

@author: Simon Wilshin, swilshin@rvc.ac.uk

This is the base object for all data containers.

FEATURE REQUEST:
Add the ability to ignore subsets of data by specifying in the YAML

'''

import conventions as conv
from directoryAware import DirectoryAware as DirectoryAware 
from tree import Leaf
from defaultable import Defaultable
from mDInterpret import getID, loadMetaData

class UnsampledBaseData(DirectoryAware,Leaf,Defaultable):
    '''
    All data objects have an associated working directory and analysis 
    directory where output should be stored.
    Derived from this class are all data classes which do not have an 
    associated capture rate (nerve block conditions or substrate conditions) 
    and all those that do via BaseData. Unlike the BaseData class we do 
    not associate this class with a file or stride frequency. It will 
    usually be used to encapsulte conditions of the experiment.
    All data elements are leaves in the data tree.
    ''' 
    def __init__(self, workingDirectory, analysisDirectory, metaData, parent):  
        if not getID(metaData) and type(self).__dict__.has_key("defaultIDExt"):
            metaData["iD"] = parent.iD + type(self).defaultIDExt
        if getID(metaData):
            Leaf.__init__(self,parent,getID(metaData))
        else:
            Leaf.__init__(self,parent)
        Defaultable.__init__(self,metaData)
        DirectoryAware.__init__(self, workingDirectory,analysisDirectory,self.metaData)   

class BaseData(UnsampledBaseData):
    '''

    Derived from this class are data objects which handle force plate data 
    or motion capture data. Any universal property of data from biomechanics 
    experiments should go in this base class
    '''


    def __init__(self, workingDirectory, analysisDirectory, metaData, parent):
        '''
        Constructor, initialise the associated file, sample rate, etc. The 
        dictionary data is where all computed quantities should be stored 
        so they can be acessed by the modified __getitem__ method below.
        '''
        UnsampledBaseData.__init__(self, workingDirectory, analysisDirectory, metaData, parent)
        loadMetaData("sampleRate",self, self.metaData,None) # Can be specified later
        loadMetaData("timeCuts",self, self.metaData,None)
        loadMetaData("fileType",self, self.metaData,conv.file)
        self.fileName=self.metaData["fileName"]
        self.totalSamples=None
        self.data=dict()
        
    def __setitem__(self, k, v):
        '''
        Virtual method to allow item assignment
        '''
        self.data[k]=v
        
    def mangleKey(self,k):
        '''
        Virtual method, when overridden should convert the key k to a
        sensible entry in the dictionary self.data. The idea is to make 
        the system robust by having it do something sensible regardless 
        of how the user request the data be given.
        Returns: A mangled version of the key k. 
        '''
        return(k)

    def has_key(self,k):
        '''
        Checks if the key k exists in the data object. The intention is to 
        allow users to query if a key exists interactively so the key is 
        mangled (projected from a list of acceptable key names on to the 
        space of actual key names in data) first. As such this should not 
        be used for the purposes of finding unique keys.
        Returns: If the key k corresponds to data in the object
        '''
        return(self.data.has_key(self.mangleKey(k)))

    def __getitem__(self,k):
        '''
        Returns an element of the dictionary self.data based on the key k 
        which is mangled (projected from a list of acceptable key names on 
        to the space of actual key names in data) first so the user has a 
        variety of sensible ways to get at the data. Will throw an exception 
        if k does not correspond to anything after mangling.
        Returns: The data associated with k. 
        '''
        return(self.data[self.mangleKey(k)])

    def keys(self):
        '''
        Returns a list of unique keys for this data object. This is a list 
        of unique keys not a list of keys including mangling options. No mangling
        (projecting from a list of acceptable key names on to the space of actual
        key names in data) is performed here. 
        '''
        return(self.data.keys())

    def hasKeyAndTrue(self,k):
        '''
        Checks whether a key exists and also that it is true; if it does not exist
        or exists and evaluates to false, returns false.
        '''
        if self.metaData.has_key(k):
            return bool(self.metaData[k])
        else:
            return False
        
if __name__=='__main__':
    '''
    No tests here since the classes here are never supposed to be instantiated.
    '''
    print "No tests for baseData, these classes should never be directly instantiated"