'''
Created on 05 Oct 2011

@author: Simon Wilshin

A experiment tree class.
'''

import inspect
from os.path import join as osjoin

import dataContainers
import analysisContainers
from tree import Branch, TreeException
from directoryAware import DirectoryAware
from mDInterpret import getID,getChildren
from defaultable import Defaultable

RESTHRESH = 100

class ExperimentBranch(Branch,DirectoryAware,Defaultable):
    def __init__(self, workingDir, analysisDir, metaData, parent):
        Branch.__init__(self,parent,getID(metaData))
        Defaultable.__init__(self,metaData)
        DirectoryAware.__init__(self,workingDir,analysisDir,self.metaData)
        self.pendingAnalyses = []
        if self.metaData.has_key("data"):
            for d in self.metaData["data"]:
                if "verbosity" in self.metaData and self.metaData["verbosity"]>0:
                    print "Adding type %s data with fileName: %s" % (d["type"],d["fileName"])
                self.addData(d)
        if self.metaData.has_key("analyses"):
            for a in self.metaData["analyses"]:
                self.addAnalysis(a)
        nchildren = getChildren(self.metaData)
        for c in nchildren:
            ExperimentBranch(self.workingDirectory, self.analysisDirectory, c, self)
        if self.parent==None:
            self.subjects = self.children
        elif self.parent.parent == None:
            self.trials = self.children

    def addData(self,metaData):
        '''
        Data entries in the tree are instatiated immediately.
        '''
        for name,obj in inspect.getmembers(dataContainers):
            if inspect.isclass(obj) and name==metaData["type"]:
                obj(self.workingDirectory, self.analysisDirectory, metaData, self)

    def addAnalysis(self,metaData):
        '''
        Analyses are not instatiated immediately, but rather 
        wait until the resolution phase of the tree
        '''
        self.pendingAnalyses.append(metaData)

    def resolvableAnalysis(self,metaData):
        '''
        Determines if an analysis that is due to be added can be resolved 
        by looking for the presence of the targets.
        '''
        targets = []
        if metaData.has_key("targets"):
            targets = metaData["targets"]
        else:
            for name,obj in inspect.getmembers(analysisContainers):
                if inspect.isclass(obj) and name==metaData["type"]:
                    targets = obj.defaultTargets
        if targets == []:
            raise analysisContainers.AnalysisError("No such Analysis, %s"%metaData["type"])
        return(all([self.canResolve(t) for t in targets]))

    def tryResolve(self):
        '''
        This method will try to resolve the analyses 
        and all sub elements of the tree, it will return 
        a list of the elements of the tree that are 
        needed in order to resolve the remaining pending 
        analyses.
        '''
        npendingAnalyses = []
        for metaData in self.pendingAnalyses:
            if self.resolvableAnalysis(metaData):
                for name,obj in inspect.getmembers(analysisContainers):
                    if inspect.isclass(obj) and name==metaData["type"]:
                        obj(self.workingDirectory, self.analysisDirectory, metaData, self)                
            else:
                npendingAnalyses.append(metaData)
        self.pendingAnalyses = npendingAnalyses
        nPendings = []
        for c in self.children:
            if type(c)==ExperimentBranch:
                nPendings = c.tryResolve()
        return(self.pendingAnalyses+nPendings)

    def keys(self):
        nkeys=[]
        for c in self.children:
            m = [o for n,o in inspect.getmembers(c) if inspect.ismethod(o) and o.__name__=="keys"]        
            if type(c)!=ExperimentBranch and m!=[]:
                nkeys.extend(m[0]())
            if type(c)==ExperimentBranch:
                nkeys.append(c.iD)
        return(sorted(list(set(nkeys))))

    def __iter__(self):
        return((c for c in self.children if type(c)==ExperimentBranch))

    def __getitem__(self,k):
        '''
        Run through the children and see if one and only one 
        provide the necessary key, if so, return that. Allows 
        one to access data at the level of the trial and not 
        concern oneself with the plug in on the level below 
        most of the time
        '''
        found = False
        for c in self.children:
            m = [o for n,o in inspect.getmembers(c) if inspect.ismethod(o) and o.__name__=="has_key"]
            if type(c)!=ExperimentBranch and m!=[] and m[0](k):
                if found:
                    raise KeyError("Resolving the request for data key %s from the tree is ambiguous in element of ID: %s."%(k,self.iD))
                else:
                    d = c[k]
                    found = True
        if found:
            return(d)
        else:
            for c in self.children:
                if type(c)==ExperimentBranch and c.iD==k:
                    return c
                    found = True
        if not found:
            raise KeyError("Key %s not found in the tree in element of ID: %s."%(k,self.iD))
    
    def callMethod(self,method,*args,**kwargs):
        retval = list()
        # Call on self:
        if method in [m for m in dir(self) if callable(getattr(self, m))]:
            retval.append((self.iD,getattr(self,method)(*args,**kwargs)))
        # Call on all children:
        for c in self.children:
            if type(c) == ExperimentBranch:
                retval.append((c.iD,c.callMethod(method,*args,**kwargs)))
            else:
                if method in [m for m in dir(c) if callable(getattr(c, m))]:
                    retval.append((c.iD,getattr(c,method)(*args,**kwargs)))
        return retval
            
class ExperimentTree(ExperimentBranch):
    def __init__(self,rootDir, metaData):
        ExperimentBranch.__init__(self,rootDir,rootDir,metaData,None)
        needed = self.tryResolve()
        unresolved = True
        resCount = 0
        while unresolved:
            resCount += 1
            if resCount == RESTHRESH:
                print "Resolution Count is at ", RESTHRESH, ". That is an anomolously large number of resolution attempts for the experiment tree."
                print "You probably have a bug in your YAML file pointing to a non-existent analysis or data container."
            nneeded = self.tryResolve()
            if nneeded == []:
                unresolved = False
            else:
                if nneeded == needed:
                    raise TreeException("Cannot resolve experiment tree! Needed %s"%str(needed),False)
                else:
                    needed = nneeded

if __name__=="__main__":
    from yaml import load as yamlload
    metaDataFile = osjoin("..",'sampleYAML','experiment.yml')
    stream = file(metaDataFile, 'r')
    metaData=yamlload(stream)
    stream.close()
    eT = ExperimentTree(osjoin("..","sampleData","dogSampleData","motionCapture"),metaData)
    '''eT = ExperimentTree(osjoin("..","sampleData","dogSampleData","motionCapture"),{"id":"testExp", "defaults":{"strideFrequencyEstimate":2.0,"maxFrequencyOfInterestMultiplier":5.0},"subjects":[{"id":"Norman","trials":[{"id":"Trial0001",
                                                                                   "data":[{"id":"0001","type":"MotionCaptureData","fileName":"gaitTransitionSampleDataDog.mat","qualisysNames":"gaitTrans_0612210_norman__cameraTest_0004_an"}],
                                                                                   "analyses":[{"id":"0001v","type":"Velocity"}]}]}]})'''
    
