'''
Created on 22 May 2012

@author: aspence
'''

import os
import numpy as np
import scipy.stats
import pylab
import utils
from scipy.io import loadmat
from scipy.interpolate import splrep, splev
from matplotlib import font_manager
from directoryAware import DirectoryAware
from dataContainers.spiderTarsiTrackData import SpiderTarsiTrackData

class SpiderGaitAblationsTrial(DirectoryAware):
    '''
    Encodes the horse nerve block trials, including data about the type of block applied and 
    the motion capture data.
    '''
    def __setitem__(self, k, v):
        '''
        Virtual method to allow item assignment
        '''
        self.data[k]=v
        
    def mangleKey(self,k):
        '''
        Virtual method, when overridden should convert the key k to a
        sensible entry in the dictionary self.data. The idea is to make 
        the system robust by having it do something sensible regardless 
        of how the user request the data be given.
        Returns: A mangled version of the key k. 
        '''
        return(k)

    def has_key(self,k):
        '''
        Checks if the key k exists in the data object. The intention is to 
        allow users to query if a key exists interactively so the key is 
        mangled (projected from a list of acceptable key names on to the 
        space of actual key names in data) first. As such this should not 
        be used for the purposes of finding unique keys.
        Returns: If the key k corresponds to data in the object
        '''
        return(self.data.has_key(self.mangleKey(k)))

    def __getitem__(self,k):
        '''
        Returns an element of the dictionary self.data based on the key k 
        which is mangled (projected from a list of acceptable key names on 
        to the space of actual key names in data) first so the user has a 
        variety of sensible ways to get at the data. Will throw an exception 
        if k does not correspond to anything after mangling.
        Returns: The data associated with k. 
        '''
        return(self.data[self.mangleKey(k)])

    def keys(self):
        '''
        Returns a list of unique keys for this data object. This is a list 
        of unique keys not a list of keys including mangling options. No mangling
        (projecting from a list of acceptable key names on to the space of actual
        key names in data) is performed here. 
        '''
        return(self.data.keys())

    def _getPosSpeedFromImgs(self):
        # Extract position and speed from a imgs sequence from srcAutoTrack.
        # Find all .mat files with tracking data:
        imgsfiles = [os.listdir(self.workingDirectory)[k] for k in range(len(os.listdir(self.workingDirectory))) if os.listdir(self.workingDirectory)[k].find('imgs-')>=0]
	imgsfiles.sort()
        # Load each and concatenate position and speed data.
        self.loc = list()
        self.ang = list()
        self.frid = list()
        for fn in imgsfiles:
            d = loadmat(os.path.join(self.workingDirectory,fn),struct_as_record=False)
            # Should store in dictionnary of keys?
            self.loc.extend([d["data"][0,k].loc[0] for k in range(d["data"].shape[1])])
            self.ang.extend([d["data"][0,k].ang[0] for k in range(d["data"].shape[1])])
            self.frid.extend([d["data"][0,k].id[0] for k in range(d["data"].shape[1])])
        self.loc = np.array(self.loc)
        self.ang = np.array(self.ang)
        self.frid = np.array(self.id)
        # Need to heavily low pass and then differentiate:
        filt = utils.FiltFiltFilter(1000,10) # Need a better filter, and non-hardcoded
        self.vel = np.diff(filt.filter(self.loc.T),axis=1).T
        self.spd = np.sqrt((self.vel**2).sum(axis=1)) # TODO: Pretty crap: find a better smoother/lowpass.
        # Let's try a spline fit. Trial and error gives a reasonable smooth:
        # These look reasonable, even have more high freq that the filffilt. Examination relative to stride
        # cutting suggests they may actually be real.
        self.locspline = np.zeros(self.loc.shape)
        self.velspline = np.zeros(self.loc.shape)
        x=range(self.loc.shape[0])
        for k in range(self.loc.shape[1]):
            tck=splrep(x,self.loc[:,k],w=np.ones((self.loc.shape[0],1)).fill(0.1),s=100)
            self.locspline[:,k] = splev(x,tck)
            self.velspline[:,k] = splev(x,tck,der=1)
        
    def computeStrideSpeeds(self):
        self.strideSpeeds=list()
        for idx0,idx1 in zip(self.spiderTarsiTrackData.strideIdx[0:-1],self.spiderTarsiTrackData.strideIdx[1:]):
            if self.getPosSpeedFromImgs:
                self.strideSpeeds.append(np.mean(self.spd[idx0:idx1]))
            else:
                self.strideSpeeds.append(np.nan)
        
    def plotPosSpeed(self):
        pylab.figure()
        ax1 = pylab.subplot2grid((4,1), (0, 0), rowspan=3)
        pylab.plot(self.loc)
        pylab.plot(self.locspline)
        pylab.axis('equal')
        pylab.grid()
        pylab.legend(('x raw','y raw', 'x spline', 'y spline'))
        pylab.title('x and y coord vs frame')
        inax=utils.insetAxes(pylab.gca())
        pylab.plot(self.loc[0:100,0])
        pylab.plot(self.locspline[0:100,0])
        pylab.title('First 100 samples X coord')
        
        ax2 = pylab.subplot2grid((4,1), (3, 0), rowspan=1)
        pylab.plot(self.vel)
        pylab.plot(self.spd)
        pylab.plot(self.velspline)
        pylab.grid()
        pylab.title('Velocities and Speed (norm vel)')
        pylab.legend(('Vx','Vy','Vmag','Vxspline','Vyspline'))
        [pylab.axvline(x=k,color='r') for k in self.spiderTarsiTrackData.strideIdx+np.array(self.spiderTarsiTrackData.trimStart)]
        pylab.ylim((-2,2))
        pylab.savefig(os.path.join(self.analysisDirectory, "bodyPosSpeed.png"))
        pylab.close('all')
        
    def __init__(self,workingDirectory, analysisDirectory, metaData):
        '''
        Initialise the base object, then add data pertaining to the 
        surface
        '''
        DirectoryAware.__init__(self, workingDirectory,analysisDirectory,metaData)
        if metaData.has_key("id"):
            self.id=metaData["id"]
        else:
            self.id="unknown"
        if metaData.has_key("trialType"):
            self.trialType=metaData["trialType"]
        else:
            self.trialType=None
        if metaData.has_key("goodtraininglegs"):
            self.goodTrainingLegs=metaData["goodtraininglegs"]
        else:
            self.goodTrainingLegs=None
        print "Processing trial ", self.id
        if metaData.has_key("getPosSpeedFromImgs"):
            self.getPosSpeedFromImgs=metaData["getPosSpeedFromImgs"]
        if self.getPosSpeedFromImgs:
            print "  Loading position and speed from imgs- file sequence."
            self._getPosSpeedFromImgs()
        self.spiderTarsiTrackData=SpiderTarsiTrackData(self.workingDirectory, self.analysisDirectory, metaData["spiderTarsiTrackData"] )
        # The above will have stride cut. Now use here to compute speeds per stride:
        # self.computeStrideSpeeds() <- NO. In case we use per leg phasers, this is now called by Experiment.
        if self.getPosSpeedFromImgs:
            self.plotPosSpeed()
