'''
Created on 9 Dec 2010

@author: Simon Wilshin, swilshin@rvc.ac.uk
'''

import numpy as np

import utils

class TemporalSignal(np.ndarray):
    ''' A signal class which keeps track of what time it is attached to 
    and when in that time frame it has data'''
    def __new__(cls, input_array, time=None,idxStart=None):
        # Input array is an already formed ndarray instance
        # We first cast to be our class type
        obj = np.asarray(input_array).view(cls)
        # add the new attribute to the created instance
        obj.time = time
        obj.idxStart=idxStart
        # Finally, we must return the newly created object:
        return obj
    
    def filter(self,sampleRate,db3,order=5,style="lowpass"):
        bf=utils.ButterFilter(sampleRate,db3,order=5,style=style)
        return(TemporalSignal(bf.filter(self),self.time,self.idxStart))
    
    def makePhase(self):
        return(TemporalSignal(utils.makePhase(self),self.time,self.idxStart))    
    
    def resample(self,ntime,method='spline',left=None,right=None):
        '''Resample up to a new time'''
        if self.ndim==1:
            t_d=np.zeros(len(ntime))
        else:
            t_d=np.zeros((self.shape[0],len(ntime)))
        t_d.fill(np.nan)
        t0=self.time[self.idxStart:]
        if t0!=ntime:
            for b in utils.nonNaNBlocksNormalForm(self.transpose()):
                tIdx=[i for i in range(len(ntime)) if ntime[i]<t0[b[1]] and ntime[i]>t0[b[0]]]
                tMinIdx=np.min(tIdx)
                tMaxIdx=np.max(tIdx)
                if self.ndim==1:
                    t_d[tMinIdx:tMaxIdx]=utils.wrappedInterpolate(t0[b[0]:b[1]],self[b[0]:b[1]],ntime[tMinIdx:tMaxIdx],method,left,right)
                else:
                    t_d[:,tMinIdx:tMaxIdx]=utils.wrappedInterpolate(t0[b[0]:b[1]],self[:,b[0]:b[1]],ntime[tMinIdx:tMaxIdx],method,left,right)
            return(TemporalSignal(t_d,ntime,0))

    def nanInterpolation(self, method='spline', left=None, right=None):
        return(TemporalSignal(utils.nanInterpolation(self, method, left, right),self.time,self.idxStart))

    def numericallyDifferentiate(self,sampleRate,frequencyOfInterest,passFactor=4,filterOrder=5):
        return(TemporalSignal(
                              utils.numericallyDifferentiate(self,sampleRate,frequencyOfInterest,passFactor,filterOrder),
                              self.time,
                              self.idxStart
                              ))
        
    def findFlatRegions(self,minl,maxVar,step):
        return(utils.findFlatRegions(self, minl, maxVar, step))
            
    
    def __getslice__(self,s0,s1):
        if s0==None:
            return(TemporalSignal(np.ndarray.__getslice__(self,s0,s1),self.time,self.idxStart))
        if (not self.__dict__.has_key("idxStart")) or self.idxStart == None:
            return(TemporalSignal(np.ndarray.__getslice__(self,s0,s1)))
        return(TemporalSignal(np.ndarray.__getslice__(self,s0,s1),self.time,self.idxStart+s0))

    def getTime(self):
        if self.ndim==1:
            return(self.time[self.idxStart:(self.idxStart+self.shape[0])])
        else:
            return(self.time[self.idxStart:(self.idxStart+self.shape[1])])

    def __array_finalize__(self, obj):
        # see InfoArray.__array_finalize__ for comments
        if obj is None: return
        self.time = getattr(obj, 'time', None)
        self.idxStart = getattr(obj, 'idxStart', None)

